@@ -186,10 +186,10 @@ vpopmail_preinit(pool_t pool, const char *args)
 
 	tmp = t_strsplit_spaces(args, " ");
 	for (; *tmp != NULL; tmp++) {
-		if (strncmp(*tmp, "cache_key=", 10) == 0) {
+		if (str_begins(*tmp, "cache_key=")) {
 			module->module.default_cache_key =
 				auth_cache_parse_key(pool, *tmp + 10);
-		} else if (strncmp(*tmp, "webmail=", 8) == 0) {
+		} else if (str_begins(*tmp, "webmail=")) {
 			if (net_addr2ip(*tmp + 8, &module->webmail_ip) < 0)
 				i_fatal("vpopmail: Invalid webmail IP address");
 		} else if (strcmp(*tmp, "blocking=no") == 0) {
